#!/bin/sh
#
# install_wheatis_node.sh
#
# Script used to install, configure and start a WheatIS node, powered by Solr.
#
# Author: F. Philippe, R. Flores
#
# Copyright INRA-URGI 2015-2018
#

CURRENT_SOLR_VERSION="6.6.2"
CORE_NAME=""
HOST_DEV="localhost"
PORT=8983
WHEATIS_NODE_ADMIN="wheatis_node_admin"
URL="https://www.apache.org/dyn/mirrors/mirrors.cgi?action=download&filename=lucene/solr/${CURRENT_SOLR_VERSION}/solr-${CURRENT_SOLR_VERSION}.tgz"
SOLR_CONF_DIR="solr/conf"
SOLR_USER="solr"
INSTALL_DIR="/opt/solr"
DATA_DIR="/var/solr"
WORK_DIR=$(mktemp -d)
INSTALL_SERVICE=false

clean (){
    EXIT_CODE="$?"
    if [ "${EXIT_CODE}" != "0" ]; then
        echo "You should remove created temp directory: ${WORK_DIR}"
    else
        rm -rf "${WORK_DIR}"
    fi
}
trap clean INT TERM EXIT

usage() {
    cat <<-EOF1
    USAGE:
        $0 -c "<CORE_NAME>" [-port <8983>] [ -install-dir <${INSTALL_DIR}> ] [ -data-dir <${DATA_DIR}> ] [--as-service]

    DESCRIPTION: 
        Script used to install, configure and start a WheatIS node, powered by Solr.
        If you want to install as a system service, you'll need to launch $0 as root or using sudo.

    PARAMS:
        -c            specify your lab's short name (mandatory). Choose it carefully since it will be visible by end-users. You may use your well-known laboratory name.
        -port         specify the port where solR will be deployed (by default 8983)
        -install-dir  the install directory of WheatIS node (need read/write permissions). Default to ${INSTALL_DIR}.
        -data-dir     the data directory of WheatIS node (need read/write permissions). Default to ${DATA_DIR}.
        --as-service   install Solr as a system service, need to execute script as root or with sudo. Currently, the script only supports CentOS, Debian, Red Hat, SUSE and Ubuntu Linux distributions. More information: https://lucene.apache.org/solr/guide/6_6/taking-solr-to-production.html#TakingSolrtoProduction-ServiceInstallationScript

	EOF1
    # ENVIRONMENT VARIABLE:
    #     If you don't want to be prompted the password to restrict Solr admin access, you may export the WHEATIS_NODE_PWD variable with your password.
  # CARE TO LET THE ABOVE TABULATION AS IS, ELSE THE EOF WON'T BE INTERPRETED CORRECTLY - http://tldp.org/LDP/abs/html/here-docs.html#LIMITSTRDASH    exit 1
}

[ $# -eq 0 ] && usage && exit 0 # exit after displaying help if no argument given

prerequesite(){
    REQUIRED="$1"
    [ -z "$1" ] && {
        echo "Required program is missing for checking. Exiting."
        exit 14
    }
    [ "$(which "${REQUIRED}" 2>/dev/null)" ] || { 
        echo "ERROR: program $REQUIRED has not been found, please add it to your path and/or install it before running the script."
        _=$((MISSING_PREREQUESITE+=1))
    }
}

MISSING_PREREQUESITE=0
check_prerequesites(){
    prerequesite "java"
    prerequesite "gpg"
    prerequesite "curl"
    prerequesite "tar"
    [ $MISSING_PREREQUESITE -gt 0 ] && {
        echo "Missing ${MISSING_PREREQUESITE} required dependencies. Please configure your environment accordingly. Now exiting."
        exit 14
    }
}
check_prerequesites

JAVA_VERSION=$(java -version 2>&1)
if ! echo "${JAVA_VERSION}" | grep -q "1.8" ; then
    echo "JAVA_VERSION must be 1.8. Please configure your environment for be able to use Java 8. Exiting."
    exit 2
fi

if [ "$(id -u)" = "0" ]; then
    echo "You should not run this script with root privileges. Do you really want to continue? (y/n)"
    read -n 1 GO_AHEAD
    if [ "${GO_AHEAD}" == "y" ]; then
        echo && echo "OK you've been informed..."
        sleep 1
    else
        echo "Exiting."
        exit 1
    fi
fi

# get params
while [ -n "$1" ]; do
    case $1 in
        -h) usage;exit 0; shift 1;;
        --help) usage;exit 0; shift 1;;
        -c) CORE_NAME=$2;CORE_NAME=$(echo "$CORE_NAME" | tr -cd '[[:alnum:]]._-');shift 2;;
        -port) PORT=$2;shift 2;;
        -install-dir) INSTALL_DIR=$2;shift 2;;
        -data-dir) DATA_DIR=$2;shift 2;;
        --as-service) INSTALL_SERVICE=true; 
            sudo_user_if_service="sudo -u ${SOLR_USER} "
            sudo_if_service="sudo "
            echo "Checking your current user (${USER}) is in sudoers to install Solr as a service..."
            if ! ${sudo_if_service} sh -c "exit 0"; then
                echo "Installing Solr as a service need to launch $0 using user in sudoers group. Exiting."
                exit 1
            fi
            shift 1;;
        --) shift;break;;
        -*) printf  "\nUnknown option: %s\n" "$1";exit 1;;
        *) break;;
    esac
done

# [ "${INSTALL_SERVICE}" != true ] && INSTALL_DIR="${INSTALL_DIR}/solr-${CURRENT_SOLR_VERSION}"

[ -z "${CORE_NAME}" ] && echo "CORE_NAME is missing, please provide the CORE_NAME that will be displayed in Solr. Exiting." && exit 1
	
SOLR_SERVER_URL_BASE="http://${HOST_DEV}:${PORT}"

check_downloaded_package(){
    echo "Checking downloaded files' signature..."
    # import Apache keys
    curl -s https://www-eu.apache.org/dist/lucene/solr/KEYS | gpg --logger-file /dev/null --import
    # import package signature
    curl -s https://www-eu.apache.org/dist/lucene/solr/${CURRENT_SOLR_VERSION}/solr-${CURRENT_SOLR_VERSION}.tgz.asc -o "${WORK_DIR}/solr-${CURRENT_SOLR_VERSION}.tgz.asc"
    # compare expected with downloaded
    gpg --logger-file /dev/null --verify "${WORK_DIR}/solr-${CURRENT_SOLR_VERSION}.tgz.asc" "${WORK_DIR}/solr-${CURRENT_SOLR_VERSION}.tgz"
    return $?
}

install_solr(){
    # check install directory is writable
    if [ ${INSTALL_SERVICE} = true ] && [ -d "${INSTALL_DIR}/solr-${CURRENT_SOLR_VERSION}" ] || [ -d "${INSTALL_DIR}" ]  ; then
        echo "Directory ${INSTALL_DIR} already exists. Skipping Solr install ... "
    else
        if ${sudo_if_service} mkdir -p "${INSTALL_DIR}"; then
            echo "Creating missing directory ${INSTALL_DIR}."
        else
            echo "${INSTALL_DIR} is not a writable directory. Exiting."
            exit 13
        fi
        if [ ${INSTALL_SERVICE} = true ]; then
            tar xzf "${WORK_DIR}/solr-${CURRENT_SOLR_VERSION}.tgz" "solr-${CURRENT_SOLR_VERSION}/bin/install_solr_service.sh" --strip-components=2 -O > "${WORK_DIR}/install_solr_service.sh"
            if ! ${sudo_if_service} sh "${WORK_DIR}/install_solr_service.sh" "${WORK_DIR}/solr-${CURRENT_SOLR_VERSION}.tgz" -f -i "${INSTALL_DIR}" -d "${DATA_DIR}" -n; then
                echo "ERROR: cannot install solr into ${INSTALL_DIR} with data into ${DATA_DIR}. Exiting."
                exit 9
            fi
            ${sudo_if_service} chown -R ${SOLR_USER}:${SOLR_USER} "${INSTALL_DIR}" "${DATA_DIR}"
        else
            # if [ ! -d "${INSTALL_DIR}" ]; then
            tar xzf "${WORK_DIR}/solr-${CURRENT_SOLR_VERSION}.tgz" -C "${INSTALL_DIR}" || {
                echo "Could not extract Solr archive, please check it is not corrupted. Exiting." 
                exit 3
            }
        fi
    fi
}

download_solr() {
    printf "Downloading Solr %s from closest mirror: %s..." "${CURRENT_SOLR_VERSION}" "${URL}"
    curl -L -H 'Accept-Encoding: gzip, deflate, br' -H 'Connection: keep-alive' --progress "${URL}" -o "${WORK_DIR}/solr-${CURRENT_SOLR_VERSION}.tgz" || {
        echo "Cannot get Solr archive from Apache servers, please check your network connection. Exiting." 
        exit 4
      }
    if ! check_downloaded_package ; then 
        echo "ERROR: downloaded package is corrupted, it does not match the Apache signature. Remove any downloaded file and restart the process."
        exit 4
    fi
}

is_solr_running() {
    sh -c "${sudo_user_if_service}${INSTALL_DIR}/solr-${CURRENT_SOLR_VERSION}/bin/solr status" | grep 'running' | grep "${PORT}" > /dev/null
    return $?
}

install_core() {
    echo "Installing core..."
    ## step to configure your own specific core
    if [ "${CORE_NAME}" ]; then
        if [ ! -d "${SOLR_CONF_DIR}" ]; then
            echo "Missing WheatIS Solr config directory !"
            exit 1
        else
            if ! is_solr_running; then
                echo "Starting solr on port ${PORT}..."
                sh -c "${sudo_user_if_service}${INSTALL_DIR}/solr-${CURRENT_SOLR_VERSION}/bin/solr start -p ${PORT}" || {
                    echo "Could not start Solr on port ${PORT}. Exiting." 
                    exit 5
                }
            fi
            # check if core already exists
            curl -fis "http://localhost:${PORT}/solr/admin/cores?action=STATUS&core=${CORE_NAME}&wt=json" | grep -q "\"${CORE_NAME}\":{\"name\":\"${CORE_NAME}\""
            if [ $? = 0 ]; then
                echo "ERROR: a core named ${CORE_NAME} already exists. Please, before continuing, use another core name or delete it using:"
                printf "\t '${sudo_user_if_service}%s/bin/solr delete -c %s -p %s' \n" "${INSTALL_DIR}/solr-${CURRENT_SOLR_VERSION}" "${CORE_NAME}" "${PORT}"
                printf "\t '${sudo_user_if_service}%s/bin/solr restart -p %s' \n" "${INSTALL_DIR}/solr-${CURRENT_SOLR_VERSION}" "${PORT}"
                echo "Exiting."
                exit 8
            fi
            #sed -i "s#<dataDir>\${solr.data.dir:.*}</dataDir>#<dataDir>\${solr.data.dir:${DATA_DIR}}</dataDir>#" "${SOLR_CONF_DIR}/solrconfig.xml"
            sh -c "${sudo_user_if_service}${INSTALL_DIR}/solr-${CURRENT_SOLR_VERSION}/bin/solr create_core -p ${PORT} -c ${CORE_NAME} -d ${SOLR_CONF_DIR}" || {
                echo "Could not create core $CORE_NAME using config directory ${SOLR_CONF_DIR} on port ${PORT}. Exiting." 
                exit 5
            }
        fi
        echo "Core ${CORE_NAME} created."
    fi
}
	
assign_solr_role_to_admin(){
    ROLE_NAME="$1"
    curl -s -o /dev/null "http://localhost:${PORT}/solr/admin/authorization" \
    -H 'Content-type:application/json' \
    -d @- <<-EOF2
        {
          "set-permission": {
            "name": "${ROLE_NAME}",
            "role": "admin"
          }
        }
	EOF2
    [ $? != 0 ] && {
        echo "ERROR when assigning role $ROLE_NAME to admin user. Exiting."
        exit 7
    }
} 

enable_security(){
    echo "Enabling security..."
    [ -z "${WHEATIS_NODE_PWD}" ] && {
        read -srp "Please enter the password to use for admin access to your WheatIS Solr node (user $WHEATIS_NODE_ADMIN):" WHEATIS_NODE_PWD
    }
    echo
    cat > "${INSTALL_DIR}/solr-${CURRENT_SOLR_VERSION}/server/solr/security.json" <<-EOF3
        {
          "authentication": {
            "class": "solr.BasicAuthPlugin"
          },
          "authorization": {
            "class": "solr.RuleBasedAuthorizationPlugin"
          }
        }
	EOF3

    sh -c "${sudo_user_if_service}${INSTALL_DIR}/solr-${CURRENT_SOLR_VERSION}/bin/solr restart -p ${PORT}"

    # Create admin user
    curl -s "http://localhost:${PORT}/solr/admin/authentication" -H "Content-type:application/json" -o /dev/null -d @- <<-EOF4
        {
          "set-user": {
            "${WHEATIS_NODE_ADMIN}": "${WHEATIS_NODE_PWD}"
          }
        }
	EOF4

  # give admin role to created user
    curl -s "http://localhost:${PORT}/solr/admin/authorization" -H 'Content-type:application/json' -o /dev/null -d @- <<-EOF5
        {
          "set-user-role": {
            "${WHEATIS_NODE_ADMIN}": [
              "admin"
            ]
          }
        }
	EOF5

    # assign all roles but read to admin user
    assign_solr_role_to_admin collection-admin-edit
    assign_solr_role_to_admin security-edit
    assign_solr_role_to_admin security-read
    assign_solr_role_to_admin schema-edit
    assign_solr_role_to_admin schema-read
    assign_solr_role_to_admin config-edit
    assign_solr_role_to_admin core-admin-read
    assign_solr_role_to_admin core-admin-edit
    assign_solr_role_to_admin config-read
    assign_solr_role_to_admin collection-admin-edit
    assign_solr_role_to_admin collection-admin-read
    # assign_solr_role_to_admin update

    echo "Security enabled."
}

index_test() {
    echo "Testing indexing sample data..."

    if ! is_solr_running; then
        sh -c "${sudo_user_if_service}${INSTALL_DIR}/solr-${CURRENT_SOLR_VERSION}/bin/solr -p ${PORT} start" ;
    fi
    sleep 2
    sh "./solr/index_solr.sh"  --stream -port "${PORT}" -c "${CORE_NAME}" -host "localhost" < "./extracting/wheatis_sample.csv" || {
        echo "Could not index sample date into ${CORE_NAME} Solr core on port ${PORT}. Exiting." 
        exit 6
    }
    printf "\nChecking sample data..."
    TEST1_VALUE_SPLIT=$(curl -s "${SOLR_SERVER_URL_BASE}/solr/${CORE_NAME}/query" -d 'q=*:*' | grep -Po '"numFound":(\d*?,|.*?[^\\\\]")' | grep -o '[0-9]*')
    if [ "${TEST1_VALUE_SPLIT}" = 0 ]; then
        echo "Something wrong happened during indexation test. We should got results."
    else
        echo "Indexation test and querying : OK"
        BOOLEAN_QUERY_TEST=1
    fi

    printf "\nDeleting sample data..."
    curl -s "${SOLR_SERVER_URL_BASE}/solr/${CORE_NAME}/update?stream.body=<delete><query>*:*</query></delete>&commit=true" > /dev/null
    TEST2_VALUE_SPLIT=$(curl -s "${SOLR_SERVER_URL_BASE}/solr/${CORE_NAME}/query" -d 'q=*:*' | grep -Po '"numFound":(\d*?,|.*?[^\\\\]")' | grep -o '[0-9]*')
    if [ "${TEST2_VALUE_SPLIT}" != 0 ]; then 
        echo "Something wrong happened during deletion test. We shouldn't have results anymore."
    else
        echo "Deletion step : OK"
        BOOLEAN_DELETE_TEST=1
    fi

    if [ $((BOOLEAN_QUERY_TEST + BOOLEAN_DELETE_TEST)) != 2 ]; then
        echo "A problem occured when trying to index and delete test data. Please report output logs if you can not figure out the problem."
    fi

    echo "Sample data indexed successfully."
    echo "Your WheatIS node should be ready to index your data."
}

if [ -f "${INSTALL_DIR}/.installation_done" ]; then
    cat "${INSTALL_DIR}/.installation_done"
    echo "If you suspect a bug in install script, you might remove file '${INSTALL_DIR}/.installation_done'. But be careful, it may broke your Solr installation if you made some customization since the installation date. Do this at your own risks."
    exit 1
fi
download_solr
install_solr
install_core
# enable_security
index_test

${sudo_user_if_service}sh -c "echo 'WARNING: installation script $0 has already finished successfully on $(date "+%Y-%m-%d at %H:%M:%S") in order to install and configure Solr for your WheatIS node. Please do not use it anymore, prefer use directly Apache Solr facilities to upgrade or maintain your Solr instance. More information can be found here: https://lucene.apache.org/solr/guide/6_6/taking-solr-to-production.html#TakingSolrtoProduction-ServiceInstallationScript' > '${INSTALL_DIR}/.installation_done'"

exit 0
